/*
 * redux-sam.js v1.2.0
 * (c) 2018-2019 Jesse Feng
 * Released under the MIT License.
 */
'use strict';

require('celia/isFunction');
require('celia/isString');
require('celia/isObject');
require('celia/forOwn');
require('./utils-fc5ba3db.js');
require('celia/isPromiseLike');
require('celia/_append');
var Sam = require('./Sam.js');
var createHelpers = require('./createHelpers.js');
require('celia/assign');
var reducer = require('./reducer.js');
var middleware = require('./middleware.js');
var install = require('./install.js');

var createStore;
var applyMiddleware;
if (typeof window !== 'undefined' && window.Redux) {
  createStore = window.Redux.createStore;
  applyMiddleware = window.Redux.applyMiddleware;
} else {
  try {
    var Redux = require('redux');
    createStore = Redux.createStore;
    applyMiddleware = Redux.applyMiddleware;
  } catch (e) { }
}

/**
 * 创建一个 redux-sam 实例用于解析state、mutations、actions和modules
 *
 * const { store, sam, mapActions, mapMutations } = createStore({
 *   state: { ... },
 *   mutations: { ... },
 *   actions: { ... }
 *   modules: { ... }
 * });
 *
 * export { store, sam, mapActions, mapMutations };
 *
 */
function createStore$1 (options, proto) {
  var sam = new Sam(options);
  var store = createStore(
    reducer(sam),
    sam.state,
    applyMiddleware(middleware(sam))
  );

  !store.dispatchAsync && (store.dispatchAsync = sam.dispatch);

  var ref = createHelpers(sam);
  ref.store = store;
  ref.sam = sam;

  // 往组件上挂载自定义函数
  install(proto, ref);

  return ref;
}

module.exports = createStore$1;
