/*
 * redux-sam.js v1.2.0
 * (c) 2018-2019 Jesse Feng
 * Released under the MIT License.
 */
import 'celia/isFunction';
import 'celia/isString';
import 'celia/isObject';
import 'celia/forOwn';
import './utils-7fa96983.js';
import 'celia/isPromiseLike';
import 'celia/_append';
import Sam from './Sam.js';
import createHelpers from './createHelpers.js';
import 'celia/assign';
import reducer from './reducer.js';
import middleware from './middleware.js';
import install from './install.js';

var createStore;
var applyMiddleware;
if (typeof window !== 'undefined' && window.Redux) {
  createStore = window.Redux.createStore;
  applyMiddleware = window.Redux.applyMiddleware;
} else {
  try {
    var Redux = require('redux');
    createStore = Redux.createStore;
    applyMiddleware = Redux.applyMiddleware;
  } catch (e) { }
}

/**
 * 创建一个 redux-sam 实例用于解析state、mutations、actions和modules
 *
 * const { store, sam, mapActions, mapMutations } = createStore({
 *   state: { ... },
 *   mutations: { ... },
 *   actions: { ... }
 *   modules: { ... }
 * });
 *
 * export { store, sam, mapActions, mapMutations };
 *
 */
function createStore$1 (options, proto) {
  var sam = new Sam(options);
  var store = createStore(
    reducer(sam),
    sam.state,
    applyMiddleware(middleware(sam))
  );

  !store.dispatchAsync && (store.dispatchAsync = sam.dispatch);

  var ref = createHelpers(sam);
  ref.store = store;
  ref.sam = sam;

  // 往组件上挂载自定义函数
  install(proto, ref);

  return ref;
}

export default createStore$1;
