/*
 * redux-sam.js v1.2.0
 * (c) 2018-2019 Jesse Feng
 * Released under the MIT License.
 */
import 'celia/isString';
import 'celia/isObject';
import 'celia/forOwn';
import { e as error } from './utils-7fa96983.js';
import assign from 'celia/assign';

function reducer(sam) {
  return function (prevState, mutation) {
    prevState = prevState || sam.state;

    var type = mutation.type;
    var payload = mutation.payload;
    if (type && type.indexOf('@@redux') === 0) {
      return prevState;
    }

    var entry = sam._mutations[type];
    if (!entry) {
      if (process.env.NODE_ENV !== 'production') {
        error(("unknown mutation type: " + type));
      }
      return;
    }

    // 是否需要返回一个新对象
    var changed = entry.reduce(function (lastest, handler) { return lastest + (handler(payload) === false ? 0 : 1); }, 0);

    var currentState = sam.state;
    sam._subscribers.forEach(function (sub) { return sub(mutation, currentState); });

    return changed ? assign({}, currentState) : currentState;
  };
}

export default reducer;
