/*
 * redux-sam.js v1.2.0
 * (c) 2018-2019 Jesse Feng
 * Released under the MIT License.
 */
'use strict';

var isFunction = require('celia/isFunction');
var isObject = require('celia/isObject');

/**
 * 安装组件实例方法和对象
 *
 * @param {Object|Function} proto
 * @param {Object} store, sam, mapActions, mapMutations
 */
function install (proto, ref) {
  var store = ref.store;
  var sam = ref.sam;
  var mapActions = ref.mapActions;
  var mapMutations = ref.mapMutations;

  if (isFunction(proto)) {
    proto = proto.prototype;
  }
  if (isObject(proto)) {
    var prototypeAccessors = {
      // 挂载store
      $store: {
        configurable: true,
        get: function get() { return store; }
      },

      // 挂载Sam
      $sam: {
        configurable: true,
        get: function get() { return sam; }
      }
    };
    Object.defineProperties(proto, prototypeAccessors);

    // 挂载 mapActions
    proto.$mapActions = function (actions, namespace) {
      return mapActions(this, actions, namespace);
    };

    // 挂载 mapMutations
    proto.$mapMutations = function (mutations, namespace) {
      return mapMutations(this, mutations, namespace);
    };
  }
}

module.exports = install;
