/*
 * redux-sam.js v1.2.0
 * (c) 2018-2019 Jesse Feng
 * Released under the MIT License.
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = global || self, global.createSamLogger = factory());
}(this, function () { 'use strict';

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isNil (value) {
    /* eslint eqeqeq: 0 */
    return value == null;
  }

  var isNil_1 = isNil;

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function loop (arr, start, end, callback) {
    for (; start < end; start++) {
      callback(arr[start], start, arr);
    }
  }

  var _loop = loop;

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function bindContext (iterator, context) {
    return context ? iterator.bind(context) : iterator;
  }

  var _bindContext = bindContext;

  // import hasOwn from './hasOwn';

  function forOwn (value, iterator, context) {
    iterator = _bindContext(iterator, context);
    var keys = Object.keys(value);
    var len = keys.length;
    for (var i = 0, key = (void 0); i < len; i++) {
      key = keys[i];
      iterator(value[key], key, value) === false && (i = len);
    }
    // for (let key in value) {
    //   if (hasOwn(value, key) && iterator(value[key], key, value) === false) {
    //     break;
    //   }
    // }
  }

  var _forOwn = forOwn;

  function _assign (target) {
    if (isNil_1(target)) {
      throw new TypeError('Cannot convert undefined or null to object');
    }
    var to = Object(target);
    _loop(arguments, 1, arguments.length, function (nextSource) {
      !isNil_1(nextSource) && _forOwn(nextSource, function (nextVal, nextKey) {
        to[nextKey] = nextVal;
      });
    });
    return to;
  }

  var _assign_1 = _assign;

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isObject (value) {
    return value !== null && typeof value === 'object';
  }

  var isObject_1 = isObject;

  var assign = Object.assign || _assign_1;

  var assign_1 = assign;

  function copy(value) {
    return isObject_1(value)
      ? assign_1({}, value)
      : value;
  }

  var isArray = Array.isArray;
  function copyDeep(value) {
    var target = value;
    if (isArray(value)) {
      target = [];
      value.forEach(function (val, i) {
        target[i] = copyDeep(val);
      });
    } else if (isObject_1(value)) {
      target = {};
      _forOwn(value, function (val, key) {
        target[key] = copyDeep(val);
      });
    }
    return target;
  }

  function looseClone (value, deep) {
    return deep ? copyDeep(value) : copy(value);
  }

  var looseClone_1 = looseClone;

  function createSamLogger(ref) {
    if ( ref === void 0 ) ref = {};
    var collapsed = ref.collapsed; if ( collapsed === void 0 ) collapsed = true;
    var filter = ref.filter; if ( filter === void 0 ) filter = function (mutation, stateBefore, stateAfter) { return true; };
    var transformer = ref.transformer; if ( transformer === void 0 ) transformer = function (state) { return state; };
    var mutationTransformer = ref.mutationTransformer; if ( mutationTransformer === void 0 ) mutationTransformer = function (mut) { return mut; };
    var logger = ref.logger; if ( logger === void 0 ) logger = console;

    return function (sam) {
      var prevState = looseClone_1(sam.state, true);

      sam.subscribe(function (mutation, state) {
        if (typeof logger === 'undefined') {
          return;
        }
        var nextState = looseClone_1(state, true);

        if (filter(mutation, prevState, nextState)) {
          var time = new Date();
          var formattedTime = " @ " + (time.toLocaleTimeString());
          var formattedMutation = mutationTransformer(mutation);
          var message = "mutation " + (mutation.type) + formattedTime;
          var startMessage = collapsed
            ? logger.groupCollapsed
            : logger.group;

          // render
          try {
            startMessage.call(logger, message);
          } catch (e) {
            console.log(message);
          }

          logger.log('%c prev state', 'color: #9E9E9E; font-weight: bold', transformer(prevState));
          logger.log('%c mutation', 'color: #03A9F4; font-weight: bold', formattedMutation);
          logger.log('%c next state', 'color: #4CAF50; font-weight: bold', transformer(nextState));

          try {
            logger.groupEnd();
          } catch (e) {
            logger.log('—— log end ——');
          }
        }

        prevState = nextState;
      });
    };
  }

  return createSamLogger;

}));
