/*
 * redux-sam.js v1.2.0
 * (c) 2018-2019 Jesse Feng
 * Released under the MIT License.
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.reduxSam = {}));
}(this, function (exports) { 'use strict';

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isFunction (value) {
    return typeof value === 'function';
  }

  var isFunction_1 = isFunction;

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isString (value) {
    return typeof value === 'string';
  }

  var isString_1 = isString;

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isObject (value) {
    return value !== null && typeof value === 'object';
  }

  var isObject_1 = isObject;

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function isNil (value) {
    /* eslint eqeqeq: 0 */
    return value == null;
  }

  var isNil_1 = isNil;

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function bindContext (iterator, context) {
    return context ? iterator.bind(context) : iterator;
  }

  var _bindContext = bindContext;

  // import hasOwn from './hasOwn';

  function forOwn (value, iterator, context) {
    iterator = _bindContext(iterator, context);
    var keys = Object.keys(value);
    var len = keys.length;
    for (var i = 0, key = (void 0); i < len; i++) {
      key = keys[i];
      iterator(value[key], key, value) === false && (i = len);
    }
    // for (let key in value) {
    //   if (hasOwn(value, key) && iterator(value[key], key, value) === false) {
    //     break;
    //   }
    // }
  }

  var _forOwn = forOwn;

  function each (value, iterator, context) {
    !isNil_1(value) && _forOwn(value, iterator, context);
  }

  var forOwn$1 = each;

  /**
   * 根据路径获取子state
   *
   * @param {Sam} sam
   * @param {Array} path
   */
  function getNestedState(sam, path) {
    var rootState = sam._state;
    return path.length
      ? path.reduce(function (state, key) { return state[key]; }, rootState)
      : rootState;
  }

  /**
   * 根据路径设置子state
   *
   * @param {Sam} sam
   * @param {Array} path
   * @param {Object} childState
   */
  function setNestedState(sam, path, childState) {
    var len = path.length;
    if (len) {
      var lastIndex = len - 1;
      var parentState = getNestedState(sam, path.slice(0, lastIndex));
      var moduleName = path[lastIndex];
      parentState[moduleName] = childState;
    } else {
      sam._state = childState;
    }
    return childState;
  }

  /**
   * 移除子state
   *
   * @param {Sam} sam
   * @param {Array} path
   */
  function removeNestedState(sam, path) {
    var lastIndex = path.length - 1;
    var parentState = getNestedState(sam, path.slice(0, lastIndex));
    var moduleName = path[lastIndex];
    delete parentState[moduleName];
  }

  /**
   * 统一参数格式为一个对象
   *
   * @param {String|Object} type
   * @param {any} payload
   * @param {Object|undefined} options
   */
  function unifyObjectStyle(type, payload, options) {
    var _type;
    if (isObject_1(type) && (_type = type.type)) {
      options = type.options;
      payload = type.payload;
      type = _type;
    }

    options = options || {};

    return { type: type, payload: payload, options: options };
  }

  /**
   * 序列化命名空间
   *
   * @param {Function} fn
   */
  function normalizeNamespace(fn) {
    return function (component, map, namespace) {
      isString_1(namespace)
        ? namespace[namespace.length - 1] !== '/' && (namespace += '/')
        : (namespace = '');
      return fn(component, map, namespace);
    };
  }

  /**
   * 把数组转换成对象再处理
   *
   * @param {Object|Array} map
   * @param {Function} callback
   */
  function normalizeMap(map, callback) {
    return Array.isArray(map)
      ? map.forEach(function (key) { return callback(key, key); })
      : forOwn$1(map, callback);
  }

  /**
   * 通过命名空间查找模块
   *
   * @param {Object} sam
   * @param {String} helper
   * @param {String} namespace
   * @return {Object}
   */
  function getContextByNamespace(sam, helper, namespace) {
    var ctx = sam._contextNamespaceMap[namespace];
    return ctx;
  }

  function isPromiseLike (value) {
    return !!value &&
      isFunction_1(value.then) &&
      isFunction_1(value.catch);
  }

  var isPromiseLike_1 = isPromiseLike;

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function append (arr, obj) {
    arr[arr.length] = obj;
    return arr;
  }

  var _append = append;

  /**
   *  挂载订阅函数
   *
   * @param {Function} fn
   * @param {Array} subs
   */
  function genericSubscribe(fn, subs) {
    if (subs.indexOf(fn) === -1) {
      _append(subs, fn);
    }
    return function () {
      var i = subs.indexOf(fn);
      if (i > -1) {
        subs.splice(i, 1);
      }
    };
  }

  /**
   * 重置函数
   * @param {Sam} sam
   * @param {Object} newModule
   */
  function resetSam(sam, newModule) {
    sam._actions = Object.create(null);
    sam._mutations = Object.create(null);
    sam._contextNamespaceMap = Object.create(null);

    // 初始化模块
    installModule(sam, [], newModule, true);
  }

  /**
   * 构建函数中需要的函数
   *
   * @param {Sam} sam
   * @param {String} namespace
   * @param {Array} path
   */
  function makeLocalContext(sam, namespace, path) {
    var hasNamespace = namespace !== '';

    var local = {
      dispatch: function dispatch(_type, _payload, _options) {
        var args = unifyObjectStyle(_type, _payload, _options);
        var payload = args.payload;
        var options = args.options;
        var type = args.type;

        if (hasNamespace && !options.root) {
          type = namespace + type;
        }
        return sam.dispatch(type, payload, options);
      },

      commit: function commit(_type, _payload, _options) {
        if (hasNamespace) {
          var args = unifyObjectStyle(_type, _payload, _options);
          var type = args.type;
          var payload = args.payload;
          var options = args.options;

          if (!options.root) {
            type = namespace + type;
          }
          return sam.commit(type, payload, options);
        } else {
          return sam.commit(_type, _payload, _options);
        }
      },

      getState: function getState() {
        return getNestedState(sam, path);
      },

      setState: function setState(ret) {
        isObject_1(ret) && setNestedState(sam, path, ret);
        return ret;
      }
    };

    return local;
  }

  /**
   * 安装模块
   *
   * @param {Sam} sam
   * @param {Array|String} path
   * @param {Object} rawModule
   * @param {Boolean} hot
   */
  function installModule(sam, path, rawModule, hot) {
    var isRoot = !path.length;

    var rawState = rawModule.state;
    var state = (isFunction_1(rawState) ? rawState() : rawState) || Object.create(null);
    var namespace = path.reduce(function (namespace, key) { return namespace + (rawModule.namespaced ? key + '/' : ''); }, '');
    var local = makeLocalContext(sam, namespace, path);

    // 判断不是顶层模块
    if (!isRoot) {
      // 分类指定命名空间的模块
      if (rawModule.namespaced) {
        sam._contextNamespaceMap[namespace] = local;
      }

      // 挂载子模块到跟模块下
      if (!hot) {
        setNestedState(sam, path, state);
      }
    } else {
      sam._state = state;
    }

    forOwn$1(rawModule.mutations, function (mutation, key) {
      var namespacedType = namespace + key;
      registerMutation(sam, namespacedType, mutation, local);
    });

    forOwn$1(rawModule.actions, function (action, key) {
      var type = action.root ? key : namespace + key;
      var handler = action.handler || action;
      registerAction(sam, type, handler, local);
    });

    forOwn$1(rawModule.modules, function (child, key) {
      installModule(sam, path.concat(key), child, hot);
    });
  }

  /**
   * 注册mutation
   * @param {Sam} sam
   * @param {String} type
   * @param {Function} handler
   * @param {Object} param3
   */
  function registerMutation(sam, type, handler, ref) {
    var setState = ref.setState;
    var getState = ref.getState;

    var entry = sam._mutations[type] || (sam._mutations[type] = []);
    handler = handler.bind(sam);

    entry.push(function (payload) { return setState(handler(getState(), payload)); });
  }

  /**
   * 注册action
   * @param {Sam} sam
   * @param {String} type
   * @param {Function} handler
   * @param {Object} param3
   */
  function registerAction(sam, type, handler, ref) {
    var dispatch = ref.dispatch;
    var commit = ref.commit;
    var getState = ref.getState;

    var entry = sam._actions[type] || (sam._actions[type] = []);
    handler = handler.bind(sam);

    entry.push(function (payload) {
      var res = handler({
        dispatch: dispatch,
        commit: commit,
        state: getState(),
        rootState: sam.state
      }, payload);
      if (!isPromiseLike_1(res)) {
        res = Promise.resolve(res);
      }
      return res;
    });
  }

  /**
   * 创建一个 redux-sam 实例用于解析state、mutations、actions和modules
   *
   * const sam = new Sam({
   *   state: { ... },
   *   mutations: { ... },
   *   actions: { ... }
   *   modules: { ... }
   * });
   *
   * const store = Redux.createStore(reducer(sam), sam.state, applyMiddleware(middleware(sam)));
   *
   */
  var Sam = function Sam(options) {
    if ( options === void 0 ) options = {};

    var plugins = options.plugins; if ( plugins === void 0 ) plugins = [];
    var sam = this;

    // 内部状态
    sam._rawRootModule = options;
    sam._state = Object.create(null);
    sam._actions = Object.create(null);
    sam._actionSubscribers = [];
    sam._mutations = Object.create(null);
    sam._subscribers = [];
    sam._contextNamespaceMap = Object.create(null);

    // 初始化state和子模块
    installModule(sam, [], options);

    // 执行插件
    plugins.forEach(function (plugin) { return isFunction_1(plugin) && plugin(sam); });
  };

  var prototypeAccessors = { state: { configurable: true } };

  prototypeAccessors.state.get = function () {
    return this._state;
  };

  prototypeAccessors.state.set = function (v) {
  };

  /**
   * 订阅追踪state
   * @param {Function} fn
   */
  Sam.prototype.subscribe = function subscribe (fn) {
    return genericSubscribe(fn, this._subscribers);
  };

  /**
   * 可以订阅action触发前后的公共处理逻辑
   * @param {Function} fn
   */
  Sam.prototype.subscribeAction = function subscribeAction (fn) {
    return genericSubscribe(
      isFunction_1(fn) ? { before: fn } : fn,
      this._actionSubscribers
    );
  };

  /**
   * 替换state，未来可能会有一些其它逻辑处理
   * @param {Object} state
   */
  Sam.prototype.replaceState = function replaceState (state) {
    this._state = state;
  };

  /**
   * 动态注册state、mutations、actions和modules，可用服务端渲染
   *
   * @param {String|Array} path
   * @param {Object} rawModule
   * @param {Object} options
   */
  Sam.prototype.registerModule = function registerModule (path, rawModule, options) {
      if ( options === void 0 ) options = {};

    if (isString_1(path)) {
      path = [path];
    }

    installModule(
      this,
      path,
      rawModule,
      options.preserveState
    );
  };

  /**
   * 动态移除已加载的模块
   *
   * @param {String|Array} path
   */
  Sam.prototype.unregisterModule = function unregisterModule (path) {
    if (isString_1(path)) {
      path = [path];
    }

    removeNestedState(this, path);

    resetSam(this, this._rawRootModule);
  };

  /**
   * 热更新时调用
   * @param {Object} newOptions
   */
  Sam.prototype.hotUpdate = function hotUpdate (newOptions) {
    resetSam(this, newOptions);
  };

  Object.defineProperties( Sam.prototype, prototypeAccessors );

  /**
   * 映射 mutations 组件的实例方法
   *
   * constructor(props) {
   *   super(props);
   *
   *   mapMutations(this, ['increment', 'decrement']);
   * }
   *
   * @param {Sam} sam
   */
  function mapMutations(sam) {
    return normalizeNamespace(function (component, mutations, namespace) {
      normalizeMap(mutations, function (val, key) {
        component[key] = function mappedMutation() {
          var args = [], len = arguments.length;
          while ( len-- ) args[ len ] = arguments[ len ];

          var commit = sam.commit;
          if (namespace) {
            var ctx = getContextByNamespace(sam, 'mapMutations', namespace);
            if (!ctx) {
              return;
            }
            commit = ctx.commit;
          }
          return isFunction_1(val)
            ? val.apply(component, [commit ].concat( args))
            : commit.apply(sam, [val ].concat( args));
        };
      });
      return component;
    });
  }
  /**
   * 映射 actions 组件的实例方法
   *
   * constructor(props) {
   *   super(props);
   *
   *   mapMutations(this, ['increment', 'decrement', 'incrementIfOdd', 'incrementAsync']);
   * }
   *
   * @param {Sam} sam
   */
  function mapActions(sam) {
    return normalizeNamespace(function (component, actions, namespace) {
      normalizeMap(actions, function (val, key) {
        component[key] = function mappedMutation() {
          var args = [], len = arguments.length;
          while ( len-- ) args[ len ] = arguments[ len ];

          var dispatch = sam.dispatch;
          if (namespace) {
            var ctx = getContextByNamespace(sam, 'mapActions', namespace);
            if (!ctx) {
              return;
            }
            dispatch = ctx.dispatch;
          }
          return isFunction_1(val)
            ? val.apply(component, [dispatch ].concat( args))
            : dispatch.apply(sam, [val ].concat( args));
        };
      });
      return component;
    });
  }
  /**
   * 创建公用函数
   * @param {Object} sam
   */
  function createHelpers (sam) {
    return {
      mapMutations: mapMutations(sam),
      mapActions: mapActions(sam)
    };
  }

  /*
   * celia.js v6.1.4
   * (c) 2018-2019 Jesse Feng
   * Released under the MIT License.
   */

  function loop (arr, start, end, callback) {
    for (; start < end; start++) {
      callback(arr[start], start, arr);
    }
  }

  var _loop = loop;

  function _assign (target) {
    if (isNil_1(target)) {
      throw new TypeError('Cannot convert undefined or null to object');
    }
    var to = Object(target);
    _loop(arguments, 1, arguments.length, function (nextSource) {
      !isNil_1(nextSource) && _forOwn(nextSource, function (nextVal, nextKey) {
        to[nextKey] = nextVal;
      });
    });
    return to;
  }

  var _assign_1 = _assign;

  var assign = Object.assign || _assign_1;

  var assign_1 = assign;

  function reducer(sam) {
    return function (prevState, mutation) {
      prevState = prevState || sam.state;

      var type = mutation.type;
      var payload = mutation.payload;
      if (type && type.indexOf('@@redux') === 0) {
        return prevState;
      }

      var entry = sam._mutations[type];
      if (!entry) {
        return;
      }

      // 是否需要返回一个新对象
      var changed = entry.reduce(function (lastest, handler) { return lastest + (handler(payload) === false ? 0 : 1); }, 0);

      var currentState = sam.state;
      sam._subscribers.forEach(function (sub) { return sub(mutation, currentState); });

      return changed ? assign_1({}, currentState) : currentState;
    };
  }

  /**
   * 触发action订阅
   * @param {Array} actionSubscribers 订阅回调函数集合
   * @param {String} method before或者after方法名
   * @param {Object} action 包括type和payload还有options
   * @param {Object} state 顶层state或者嵌套子state
   */
  function triggerActionSubscribers(actionSubscribers, method, action, state) {
    try {
      actionSubscribers
        .forEach(function (sub) {
          var fn = sub[method];
          fn && fn(action, state);
        });
    } catch (e) {
    }
  }

  /**
   * redux中间件
   * @param {Sam} sam
   */
  function middleware(sam) {
    return function (ref) {
      var dispatch = ref.dispatch;

      // 挂载 commit 方法 至 sam 对象上，用于触发异步函数
      !sam.dispatch && (sam.dispatch = function (_type, _payload, _options) {
        var action = unifyObjectStyle(_type, _payload, _options);
        var options = action.options;
        options.async === undefined && (options.async = true);
        return dispatch(action);
      });

      return function (next) {
        // 挂载 commit 方法 至 sam 对象上，用于同步更新状态
        !sam.commit && (sam.commit = function (_type, _payload, _options) {
          // 入参兼容
          return next(unifyObjectStyle(_type, _payload, _options));
        });

        return function (_type, _payload, _options) {
          // 入参兼容
          var action = unifyObjectStyle(_type, _payload, _options);
          var options = action.options;

          // 触发异步方法
          if (options.async) {
            var type = action.type;
            var payload = action.payload;
            var _actions = sam._actions;
            var _actionSubscribers = sam._actionSubscribers;

            var entry = _actions[type];
            if (!entry) {
              // return;
              entry = [function () {
                return new Promise(function (resolve) {
                  resolve(next(action));
                });
              }];
            }

            triggerActionSubscribers(_actionSubscribers, 'before', action, sam.state);

            var result = entry.length > 1
              ? Promise.all(entry.map(function (handler) { return handler(payload); }))
              : entry[0](payload);

            return result.then(function (res) {
              triggerActionSubscribers(_actionSubscribers, 'after', action, sam.state);
              return res;
            });
          }

          // next 为 redux createStore 里面的 dispatch 方法
          return next(action);
        };
      };
    };
  }

  /**
   * 安装组件实例方法和对象
   *
   * @param {Object|Function} proto
   * @param {Object} store, sam, mapActions, mapMutations
   */
  function install (proto, ref) {
    var store = ref.store;
    var sam = ref.sam;
    var mapActions = ref.mapActions;
    var mapMutations = ref.mapMutations;

    if (isFunction_1(proto)) {
      proto = proto.prototype;
    }
    if (isObject_1(proto)) {
      var prototypeAccessors = {
        // 挂载store
        $store: {
          configurable: true,
          get: function get() { return store; }
        },

        // 挂载Sam
        $sam: {
          configurable: true,
          get: function get() { return sam; }
        }
      };
      Object.defineProperties(proto, prototypeAccessors);

      // 挂载 mapActions
      proto.$mapActions = function (actions, namespace) {
        return mapActions(this, actions, namespace);
      };

      // 挂载 mapMutations
      proto.$mapMutations = function (mutations, namespace) {
        return mapMutations(this, mutations, namespace);
      };
    }
  }

  var createStore;
  var applyMiddleware;
  if (typeof window !== 'undefined' && window.Redux) {
    createStore = window.Redux.createStore;
    applyMiddleware = window.Redux.applyMiddleware;
  } else {
    try {
      var Redux = require('redux');
      createStore = Redux.createStore;
      applyMiddleware = Redux.applyMiddleware;
    } catch (e) { }
  }

  /**
   * 创建一个 redux-sam 实例用于解析state、mutations、actions和modules
   *
   * const { store, sam, mapActions, mapMutations } = createStore({
   *   state: { ... },
   *   mutations: { ... },
   *   actions: { ... }
   *   modules: { ... }
   * });
   *
   * export { store, sam, mapActions, mapMutations };
   *
   */
  function createStore$1 (options, proto) {
    var sam = new Sam(options);
    var store = createStore(
      reducer(sam),
      sam.state,
      applyMiddleware(middleware(sam))
    );

    !store.dispatchAsync && (store.dispatchAsync = sam.dispatch);

    var ref = createHelpers(sam);
    ref.store = store;
    ref.sam = sam;

    // 往组件上挂载自定义函数
    install(proto, ref);

    return ref;
  }

  var version = '1.2.0';

  exports.Sam = Sam;
  exports.createHelpers = createHelpers;
  exports.createStore = createStore$1;
  exports.install = install;
  exports.middleware = middleware;
  exports.reducer = reducer;
  exports.version = version;

}));
